﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Padaszok_
{
    /// <summary>
    /// Két fa állapot összehasonlítása
    /// </summary>
    public class FaHasonlit
    {
        /// <summary>
        /// Két file/mappa lehetséges változásai
        /// </summary>
        public enum Kulonbseg
        {
            UGYAN,
            LETR,
            MOD,
            TOROL,
            RTOROL
        }

        /// <summary>
        /// Különbségek tárolására szolgáló adat
        /// </summary>
        public class KulElem
        {
            public Kulonbseg Kul { get; private set; }
            public FileElem UjFile { get; private set; }
            public KulElem(Kulonbseg kul, FileElem ujfile)
            {
                Kul = kul;
                UjFile = ujfile;
            }

            public override string ToString()
            {
                string ret = "";
                switch (Kul)
                {
                    case Kulonbseg.RTOROL:
                        ret += "REKURZIV-TOROL " + UjFile.TeljUt;
                        break;
                    case Kulonbseg.TOROL:
                        ret += "TOROL " + UjFile.TeljUt;
                        break;
                    case Kulonbseg.LETR:
                        ret += "LETREHOZ " + UjFile.ToString();
                        break;
                    case Kulonbseg.MOD:
                        ret += "MODOSIT " + UjFile.ToString();
                        break;
                    case Kulonbseg.UGYAN:
                        ret += "UGYANAZ " + UjFile.ToString();
                        break;
                }
                return ret;
            }
        }

        /// <summary>
        /// Két állapot összehasonlítása
        /// </summary>
        /// <param name="regi">Régi állapot</param>
        /// <param name="uj">Új állapot</param>
        /// <returns>A különbségek listája</returns>
        public List<KulElem> Hasonlit(FaElem regi, FaElem uj)
        {
            List<KulElem> diff = new List<KulElem>();
            int ri = 0;
            int ui = 0;
            //Amíg van vizsgálatlan file
            while (ri < regi.Gyerekek.Count || ui < uj.Gyerekek.Count)
            {
                //Amennyiben a régi állapot elfogyott
                //Minden új
                if (ri == regi.Gyerekek.Count)
                {
                    FaElem ujgyer = uj.Gyerekek[ui];
                    diff.AddRange(AddUjat(ujgyer));
                    ++ui;
                }
                //Amennyiben az új állapot elfogyott
                //Mindent törlünk
                else if (ui == uj.Gyerekek.Count)
                {
                    FaElem rgyer = regi.Gyerekek[ri];
                    diff.Add(OkosTorol(rgyer));
                    ++ri;
                }
                else
                {
                    FaElem rgyer = regi.Gyerekek[ri];
                    FaElem ugyer = uj.Gyerekek[ui];
                    switch (rgyer.Nev.Kulonbozik(ugyer.Nev)) //Adott elemek összehasonlítása
                    {
                        case FileElem.FileKulonbseg.UGYANAZ: //Egyezés
                            diff.Add(new KulElem(Kulonbseg.UGYAN, rgyer.Nev));
                            if (rgyer.Nev.KonyvtarE())
                            {
                                diff.AddRange(Hasonlit(rgyer, ugyer));
                            }
                            ri += 1;
                            ui += 1;
                            break;
                        case FileElem.FileKulonbseg.MODOSULT: //Módosítás
                            diff.Add(new KulElem(Kulonbseg.MOD, ugyer.Nev));
                            if (rgyer.Nev.KonyvtarE())
                            {
                                diff.AddRange(Hasonlit(rgyer, ugyer));
                            }
                            ri += 1;
                            ui += 1;
                            break;
                        case FileElem.FileKulonbseg.MAS_FILE: //Más
                            if(rgyer.Nev.TeljUt.CompareTo(ugyer.Nev.TeljUt) < 0) //Ha eltűnt a régi
                            {
                                diff.Add(OkosTorol(rgyer)); //Törlés
                                ++ri;
                            }
                            else //Ha új jelent meg
                            {
                                diff.AddRange(AddUjat(ugyer)); //Hozzáadás
                                ++ui;
                            }
                            break;
                    }
                }
            }
            return diff;
        }

        /// <summary>
        /// Mapparendszer törlése
        /// </summary>
        /// <param name="regi">Törlendő</param>
        /// <returns></returns>
        private KulElem OkosTorol(FaElem regi)
        {
            if (regi.Nev.KonyvtarE()) //Könyvtárat rekurzívan
            {
                return new KulElem(Kulonbseg.RTOROL, regi.Nev);
            }
            else //Filet egyszerűen
            {
                return new KulElem(Kulonbseg.TOROL, regi.Nev);
            }
        }

        /// <summary>
        /// Új mapparendszer hozzáadása
        /// </summary>
        /// <param name="uj">A hozzáadandó adatok</param>
        /// <returns></returns>
        private List<KulElem> AddUjat(FaElem uj)
        {
            List<KulElem> diff = new List<KulElem>();
            diff.Add(new KulElem(Kulonbseg.LETR, uj.Nev)); //File/mappa létrehozása
            if (uj.Nev.KonyvtarE()) //Mappa beli fileok rekurzív létrehozása
            {
                foreach (FaElem elem in uj.Gyerekek)
                {
                    diff.AddRange(AddUjat(elem));
                }
            }
            return diff;
        }
    }
}
