﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Padaszok_
{
    /// <summary>
    /// Filelal kapcsolatos adatok
    /// </summary>
    public class FileElem
    {
        public string TeljUt { get; private set; }
        public string[] Konyvtar { get; private set; }
        public string FileNev { get; private set; }
        public int Meret { get; private set; }
        public int ModositIdo { get; private set; }

        /// <summary>
        /// Név, méret, mappaszerkezetbeli hely megállapítása
        /// </summary>
        /// <param name="sor">Az adatfile nyers sora</param>
        public FileElem(string sor)
        {
            string[] sorelemek = sor.Split(' ');
            TeljUt = sorelemek[0];
            Konyvtar = sorelemek[0].Split('/');
            if (KonyvtarE())
            {
                FileNev = Konyvtar[Konyvtar.Length - 2];
            }
            else
            {
                FileNev = Konyvtar[Konyvtar.Length - 1];
            }
            Meret = int.Parse(sorelemek[1]);
            ModositIdo = int.Parse(sorelemek[2]);
        }


        /// <summary>
        /// Meghatározza egy fileról hogy speciális-e
        /// </summary>
        /// <returns></returns>
        public bool SpecialisE()
        {
            if (KonyvtarE()) return false;
            if (Konyvtar.Last()[0] == '.') return true;
            if (Konyvtar.Last().EndsWith(".tmp")) return true;
            if (Meret > 104857600) return true; // 104857600 == 100*2^20

            return false;
        }

        /// <summary>
        /// Meghatározza hogy könyvár-e
        /// </summary>
        /// <returns></returns>
        public bool KonyvtarE()
        {
            return Konyvtar.Last() == "";
        }

        /// <summary>
        /// Lehetséges különbségek
        /// </summary>
        public enum FileKulonbseg
        {
            UGYANAZ,
            MAS_FILE,
            MODOSULT
        }

        /// <summary>
        /// Különbség megállapítása egy másik filetól
        /// </summary>
        /// <param name="masik">A másik file</param>
        /// <returns>Különbség</returns>
        public FileKulonbseg Kulonbozik(FileElem masik)
        {
            if (TeljUt != masik.TeljUt) return FileKulonbseg.MAS_FILE; //Más hely/név
            if (Meret != masik.Meret || ModositIdo != masik.ModositIdo) //Módosult adatok
                return FileKulonbseg.MODOSULT;
            return FileKulonbseg.UGYANAZ;
        }

        /// <summary>
        /// Kiírásra előkészítés
        /// </summary>
        /// <returns>Kiírásra kerülő szöveg</returns>
        public override string ToString()
        {
            return TeljUt + " " + Meret.ToString() + " " + ModositIdo.ToString();
        }

        public override bool Equals(object obj)
        {
            if (obj is FileElem)
            {
                return Kulonbozik(obj as FileElem) == FileKulonbseg.UGYANAZ;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return Konyvtar.GetHashCode() +
                Meret.GetHashCode() + ModositIdo.GetHashCode();
        }
    }
}
